#define _BSD_SOURCE

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>

#include "proto.h"

#define MAX_LINE 100
#define MAX_ARGS 10

//#define DELAY() usleep(100000)
#define DELAY()

int config_tty(int fd, struct termios *oldtio)
{
	int failed = 0;
	struct termios newtio;

	/* store old I/O settings, if requested */
	if (oldtio) {
		if (tcgetattr(fd, oldtio) == -1) {
			perror("error getting device attributes");
			failed = 1;
			goto exit;
		}
	}

	/* configure new terminal I/O settings */
	memset(&newtio, 0, sizeof(newtio));
	newtio.c_cflag = B9600 | CS8 | CLOCAL | CREAD;
	newtio.c_iflag = IGNPAR;
	newtio.c_oflag = 0;
	newtio.c_lflag = 0;
	newtio.c_cc[VMIN] = 1;

	/* discard unread data */
	tcflush(fd, TCIFLUSH);

	/* set new I/O settings */
	if (tcsetattr(fd, TCSANOW, &newtio) == -1) {
		perror("error setting device attributes");
		failed = 1;
		goto exit;
	}

exit:
	return failed ? -1 : 0;
}

void restore_tty(int fd, struct termios *tio)
{
	if (tcsetattr(fd, TCSANOW, tio) == -1)
		perror("error restoring device attributes");
}

void send_frame(int fd, struct frame *f)
{
	DELAY();
	write(fd, &f->type, 1);
	for (int n = 0; n < MAX_PARAM_LEN; n++) {
		DELAY();
		write(fd, &f->param[n], 1);
	}
}

void recv_frame(int fd, struct frame *f)
{
	uint8_t type;
	read(fd, &type, 1);
	f->type = type;

	for (int n = 0; n < MAX_PARAM_LEN; n++) {
		read(fd, &f->param[n], 1);
	}
}

// chan, pwm
void cmd_set_pwm(int fd, int argc, char *argv[])
{
	struct frame f;
	struct param_set_pwm *p = (struct param_set_pwm*) f.param;

	if (argc < 2) {
		printf("too few parameters\n");
		return;
	}

	f.type = CMD_SET_PWM;
	p->chan = atoi(argv[0]);
	p->pwm = atoi(argv[1]);

	send_frame(fd, &f);

	printf("target response: ");
	recv_frame(fd, &f);
	switch (f.type) {
	case RES_POSITIVE:
		printf("positive\n");
		break;
	case RES_NEGATIVE:
		printf("negative\n");
		break;
	default:
		printf("unknown\n");
		break;
	}
}

// chan, volt
void cmd_set_volt(int fd, int argc, char *argv[])
{
	struct frame f;
	struct param_set_volt *p = (struct param_set_volt*) f.param;

	if (argc < 2) {
		printf("too few parameters\n");
		return;
	}

	f.type = CMD_SET_VOLT;
	p->chan = atoi(argv[0]);
	p->volt = atoi(argv[1]);

	send_frame(fd, &f);

	printf("target response: ");
	recv_frame(fd, &f);
	switch (f.type) {
	case RES_POSITIVE:
		printf("positive\n");
		break;
	case RES_NEGATIVE:
		printf("negative\n");
		break;
	default:
		printf("unknown\n");
		break;
	}
}

// chan, index, xval, yval
void cmd_set_data(int fd, int argc, char *argv[])
{
	struct frame f;
	struct param_set_data *p = (struct param_set_data*) f.param;

	if (argc < 4) {
		printf("too few parameters\n");
		return;
	}

	f.type = CMD_SET_DATA;
	p->chan = atoi(argv[0]);
	p->index = atoi(argv[1]);
	p->xval = atoi(argv[2]);
	p->yval = atoi(argv[3]);

	send_frame(fd, &f);

	printf("target response: ");
	recv_frame(fd, &f);
	switch (f.type) {
	case RES_POSITIVE:
		printf("positive\n");
		break;
	case RES_NEGATIVE:
		printf("negative\n");
		break;
	default:
		printf("unknown\n");
		break;
	}
}

// chan, index
// xval, yval
void cmd_get_data(int fd, int argc, char *argv[])
{
	struct frame f;
	struct param_get_data *p = (struct param_get_data*) f.param;

	if (argc < 2) {
		printf("too few parameters\n");
		return;
	}

	f.type = CMD_GET_DATA;
	p->chan = atoi(argv[0]);
	p->index = atoi(argv[1]);

	send_frame(fd, &f);

	printf("target response: ");
	recv_frame(fd, &f);
	struct result_get_data *r = (struct result_get_data*) f.param;
	switch (f.type) {
	case RES_POSITIVE:
		printf("positive\n");
		printf("xval = %u\n", r->xval);
		printf("yval = %u\n", r->yval);
		break;
	case RES_NEGATIVE:
		printf("negative\n");
		break;
	default:
		printf("unknown\n");
		break;
	}
}

void cmd_load(int fd)
{
	struct frame f;

	f.type = CMD_LOAD_DATA;
	send_frame(fd, &f);

	printf("target response: ");
	recv_frame(fd, &f);
	switch (f.type) {
	case RES_POSITIVE:
		printf("positive\n");
		break;
	case RES_NEGATIVE:
		printf("negative\n");
		break;
	default:
		printf("unknown\n");
		break;
	}
}

void cmd_store(int fd)
{
	struct frame f;

	f.type = CMD_STORE_DATA;
	send_frame(fd, &f);

	printf("target response: ");
	recv_frame(fd, &f);
	switch (f.type) {
	case RES_POSITIVE:
		printf("positive\n");
		break;
	case RES_NEGATIVE:
		printf("negative\n");
		break;
	default:
		printf("unknown\n");
		break;
	}
}

void split_line(char *str, int *argc, char *argv[])
{
	int max = *argc;
	int pos = 0;

	while (*str) {
		while (*str && isspace(*str))
			str++;
		
		if (pos < max)
			argv[pos++] = str;

		while (*str && !isspace(*str))
			str++;

		if (*str) {
			*str = '\0';
			str++;
		}
	}

	*argc = pos;
}

void main_loop(int fd)
{
	char line[MAX_LINE];
	char *argv[MAX_ARGS];
	int argc;
	int running = 1;

	while (running) {
		printf("> ");
		fflush(stdout);
		fgets(line, sizeof(line), stdin);

		argc = MAX_ARGS;
		split_line(line, &argc, argv);

		if (argc > 0) {
			if (!strcmp(argv[0], "quit")) {
				running = 0;
				continue;
			}

			if (!strcmp(argv[0], "set_pwm")) {
				cmd_set_pwm(fd, argc - 1, argv + 1);
				continue;
			}

			if (!strcmp(argv[0], "set_volt")) {
				cmd_set_volt(fd, argc - 1, argv + 1);
				continue;
			}

			if (!strcmp(argv[0], "set_data")) {
				cmd_set_data(fd, argc - 1, argv + 1);
				continue;
			}

			if (!strcmp(argv[0], "get_data")) {
				cmd_get_data(fd, argc - 1, argv + 1);
				continue;
			}

			if (!strcmp(argv[0], "load")) {
				cmd_load(fd);
				continue;
			}

			if (!strcmp(argv[0], "store")) {
				cmd_store(fd);
				continue;
			}

			printf("unknown command: %s\n", argv[0]);
		}
		
	}
}

int main(int argc, char *argv[])
{
	int fd = -1;
	int failed = 0;
	struct termios oldtio;
	int oldtio_valid = 0;

	if (argc != 2) {
		printf("usage: ee_curve <device>\n");
		failed = 1;
		goto exit;
	}

	if ((fd = open(argv[1], O_RDWR | O_NOCTTY)) == -1) {
		perror("open device failed");
		failed = 1;
		goto exit;
	}

	if (config_tty(fd, &oldtio) == -1) {
		failed = 1;
		goto exit;
	}
	oldtio_valid = 1;

	main_loop(fd);

exit:
	if (oldtio_valid)
		restore_tty(fd, &oldtio);

	if (fd != -1)
		close(fd);

	return failed ? EXIT_FAILURE : EXIT_SUCCESS;
}

