// warning C4100: 'xxx': Unreferenzierter formaler Parameter
#pragma warning(disable: 4100)

#include <windows.h>

#include "resource.h"
#include "util.h"
#include "target.h"

#include "calibdlg.h"

// Handle to device (passed by main dialog)
static HANDLE hDevice;

struct DataBinding {
	int  iChannel;     // Channel
	int  iIndex;       // Data point index
	UINT uCtrlIdVolt;  // Control ID for voltage
	UINT uCtrlIdPwm;   // Control ID for PWM value
};

static const struct DataBinding BindingTable[] = {
	/* Channel       Index   Control ID for voltage    Control ID for PWM value */
	{  PARAM_CHAN_A,     0,  IDC_CALIB_EDT_CH1_VOLT1,  IDC_CALIB_EDT_CH1_PWM1,  },
	{  PARAM_CHAN_A,     1,  IDC_CALIB_EDT_CH1_VOLT2,  IDC_CALIB_EDT_CH1_PWM2,  },
	{  PARAM_CHAN_A,     2,  IDC_CALIB_EDT_CH1_VOLT3,  IDC_CALIB_EDT_CH1_PWM3,  },
	{  PARAM_CHAN_A,     3,  IDC_CALIB_EDT_CH1_VOLT4,  IDC_CALIB_EDT_CH1_PWM4,  },
	{  PARAM_CHAN_A,     4,  IDC_CALIB_EDT_CH1_VOLT5,  IDC_CALIB_EDT_CH1_PWM5,  },
	{  PARAM_CHAN_B,     0,  IDC_CALIB_EDT_CH2_VOLT1,  IDC_CALIB_EDT_CH2_PWM1,  },
	{  PARAM_CHAN_B,     1,  IDC_CALIB_EDT_CH2_VOLT2,  IDC_CALIB_EDT_CH2_PWM2,  },
	{  PARAM_CHAN_B,     2,  IDC_CALIB_EDT_CH2_VOLT3,  IDC_CALIB_EDT_CH2_PWM3,  },
	{  PARAM_CHAN_B,     3,  IDC_CALIB_EDT_CH2_VOLT4,  IDC_CALIB_EDT_CH2_PWM4,  },
	{  PARAM_CHAN_B,     4,  IDC_CALIB_EDT_CH2_VOLT5,  IDC_CALIB_EDT_CH2_PWM5,  },
};
#define NUM_DATA_BINDINGS (sizeof(BindingTable)/sizeof(BindingTable[0]))

// Get data points for specified channel (read data from target)
static BOOL OnBtnGet(HWND hWnd, int iChannel)
{
	BOOL fHandled = FALSE;
	const struct DataBinding *p;
	UINT uVolt;
	UINT uPwm;
	
	for (p = BindingTable; p < &BindingTable[NUM_DATA_BINDINGS]; p++) {
		// skip entries for other channels
		if (p->iChannel != iChannel)
			continue;
		// Query data from target
		TargetGetData(hDevice, p->iChannel, p->iIndex, &uVolt, &uPwm);
		// Store data in controls
		SetDlgItemInt(hWnd, p->uCtrlIdVolt, uVolt, FALSE);
		SetDlgItemInt(hWnd, p->uCtrlIdPwm, uPwm, FALSE);

		fHandled = TRUE;
	}

	return fHandled;
}

// Set data points for specified channel (write data to target)
static BOOL OnBtnSet(HWND hWnd, int iChannel)
{
	BOOL fHandled = FALSE;
	const struct DataBinding *p;
	UINT uVolt;
	UINT uPwm;
	BOOL fVoltTranslated;
	BOOL fPwmTranslated;

	for (p = BindingTable; p < &BindingTable[NUM_DATA_BINDINGS]; p++) {
		// skip entries for other channels
		if (p->iChannel != iChannel)
			continue;
		// Query data from controls
		uVolt = GetDlgItemInt(hWnd, p->uCtrlIdVolt, &fVoltTranslated, FALSE);
		uPwm = GetDlgItemInt(hWnd, p->uCtrlIdPwm, &fPwmTranslated, FALSE);
		// Write data to target (if conversion successful)
		if (fVoltTranslated && fPwmTranslated)
			TargetSetData(hDevice, p->iChannel, p->iIndex, uVolt, uPwm);

		fHandled = TRUE;
	}

	return fHandled;
}

// Loader data points from EEPROM
static BOOL OnBtnLoad(HWND hWnd)
{
	TargetLoad(hDevice);
	return TRUE;
}

// Store data points in EEPROM
static BOOL OnBtnStore(HWND hWnd)
{
	TargetStore(hDevice);
	return TRUE;
}

// Initialization of dialog
static BOOL OnInitDialog(HWND hWnd, LPVOID lpParam)
{
	// Center window on primary desktop
	CenterWindow(hWnd);
	// Store device handle
	hDevice = (HANDLE) lpParam;
	return TRUE;
}

// Button was pressed
static BOOL OnCommand(HWND hWnd, int iCtrlCode)
{
	BOOL fHandled = FALSE;

	switch (iCtrlCode) {
	case IDC_CALIB_BTN_CH1_GET:
		fHandled = OnBtnGet(hWnd, PARAM_CHAN_A);
		break;
	case IDC_CALIB_BTN_CH1_SET:
		fHandled = OnBtnSet(hWnd, PARAM_CHAN_A);
		break;
	case IDC_CALIB_BTN_CH2_GET:
		fHandled = OnBtnGet(hWnd, PARAM_CHAN_B);
		break;
	case IDC_CALIB_BTN_CH2_SET:
		fHandled = OnBtnSet(hWnd, PARAM_CHAN_B);
		break;
	case IDC_CALIB_BTN_LOAD:
		fHandled = OnBtnLoad(hWnd);
		break;
	case IDC_CALIB_BTN_STORE:
		fHandled = OnBtnStore(hWnd);
		break;
	case IDC_CALIB_BTN_CLOSE:
		SendMessage(hWnd, WM_CLOSE, 0, 0);
		fHandled = TRUE;
		break;
	}

	return fHandled;
}

// Dialog was closed
static BOOL OnClose(HWND hWnd)
{
	EndDialog(hWnd, 0);
	return TRUE;
}

INT_PTR CALLBACK CalibDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	INT_PTR iResult = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
		iResult = OnInitDialog(hWnd, (LPVOID) lParam);
		break;
	case WM_COMMAND:
		iResult = OnCommand(hWnd, LOWORD(wParam));
		break;
	case WM_CLOSE:
		iResult = OnClose(hWnd);
		break;
	}

	return iResult;
}
