/*
 * Copyright (c) 2012, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <avr/io.h>
#include <string.h>
#include <ctype.h>
#include <stdint.h>
#include <stdbool.h>

#include "pwm.h"
#include "usart.h"
#include "response.h"

#define MAX_LINE  20
#define RES_POS   true
#define RES_NEG   false

enum {
	PARSE_OK,
	PARSE_INVALID_COMMAND,
	PARSE_INVALID_PARAM,
	PARSE_MISSING_PARAM,
};

int parse_line(const uint8_t *s, uint8_t *chan, uint8_t *dc)
{
	char cmd[MAX_LINE];
	char *p;
	bool dc_found = false;

	// trim leading spaces
	while ((*s != '\0') && isspace(*s))
		s++;
		
	// read command
	p = cmd;
	while ((*s != '\0') && isalpha(*s))
		*p++ = tolower(*s++);
	*p = '\0';

	// only "set" implemented
	if (strcmp(cmd, "set") != 0)
		return PARSE_INVALID_COMMAND;

	// skip spaces
	while ((*s != '\0') && isspace(*s))
		s++;

	// read channel
	if (*s != '\0') {
		uint8_t c = isalpha(*s) ? tolower(*s) : '\0';
		s++;

		switch (c) {
		case 'a':
			*chan = PWM_CHANNEL_A;
			break;
		case 'b':
			*chan = PWM_CHANNEL_B;
			break;
		default:
			return PARSE_INVALID_PARAM;
		}
	} else {
		return PARSE_MISSING_PARAM;
	}

	// skip spaces
	while ((*s != '\0') && isspace(*s))
		s++;

	// read duty cycle
	*dc = 0;
	while ((*s != '\0') && isdigit(*s)) {
		*dc *= 10;
		*dc += *s - '0';
		dc_found = true;
		s++;
	}

	return dc_found ? PARSE_OK : PARSE_MISSING_PARAM;
}

int main(void)
{
	uint8_t chan;
	uint8_t dc;
	uint8_t line[MAX_LINE];

	pwm_init();
	usart_init();

	resp_pos(RESP_VOLTSRC_READY);

	for (;;) {
		usart_rx_line(line, MAX_LINE);
		switch (parse_line(line, &chan, &dc)) {
		case PARSE_OK:
			pwm_set_dc(chan, dc);
			resp_pos(RESP_CHANNEL_SET);
			break;
		case PARSE_INVALID_COMMAND:
			resp_neg(RESP_INVALID_CMD);
			break;
		case PARSE_INVALID_PARAM:
			resp_neg(RESP_INVALID_PARAM);
			break;
		case PARSE_MISSING_PARAM:
			resp_neg(RESP_MISSING_PARAM);
			break;
		}
		
	}
}

